#include "target.h"
#include "error.h"
#include "intl.h"
#include "log.h"
#include "util.h"

#include <dirent.h>
#include <libgen.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

bool target_is_valid(target_t *t) {
  if (NULL == t->name) {
    error_set(_("Configuration target is missing a name"));
    return false;
  }

  if (NULL == t->dst) {
    error_set(_("Configuration target \"%s\" does not have \"dst\" field"), t->name);
    return false;
  }

  if (NULL == t->src) {
    error_set(_("Configuration target \"%s\" does not have \"src\" field"), t->name);
    return false;
  }

  return true;
}

bool target_copy_dir(char *src, char *dst) {
  if (!mksubdirsp(dst, 0755))
    return false;

  DIR           *dir = opendir(src);
  struct dirent *ent = NULL;

  if (NULL == dir) {
    error_set(_("Failed to open the directory: %s"), src);
    errno = OpendirFail;
    return false;
  }

  size_t src_len = strlen(src);
  size_t dst_len = strlen(dst);

  while ((ent = readdir(dir)) != NULL) {
    if (eq(ent->d_name, ".") || eq(ent->d_name, ".."))
      continue;

    size_t len = strlen(ent->d_name);
    char   src_fp[src_len + len + 2], dst_fp[dst_len + len + 2];

    join(src_fp, src, ent->d_name);
    join(dst_fp, dst, ent->d_name);

    if (is_dir(src_fp)) {
      if (!target_copy_dir(src_fp, dst_fp))
        return false;
      continue;
    }

    if (!copyfile(src_fp, dst_fp))
      return false;
  }

  return true;
}

bool target_copy(target_t *t, bool fromdst) {
  char *src = t->src, *dst = t->dst;
  bool  ret = false;

  if (fromdst && t->nogen)
    return true;

  if (fromdst) {
    src = t->dst;
    dst = t->src;
  }

  if (!exists(src)) {
    error_set(_("Source directory \"%s\" does not exist"), src);
    errno = TargetSrcFail;
    return ret;
  }

  if (is_dir(src))
    return target_copy_dir(src, dst);

  char *dstcp  = strdup(dst);
  char *dstdir = dirname(dst);

  if (!mksubdirsp(dstdir, 0755))
    return ret;

  ret = copyfile(src, dstcp);
  free(dstcp);
  return ret;
}

void target_init(target_t *t) {
  clist_init(&t->require);
  t->next = NULL;
  t->name = NULL;
  t->desc = NULL;
  t->dst  = NULL;
  t->src  = NULL;
}

void target_free(target_t *t) {
  free(t->name);
  free(t->desc);
  free(t->dst);
  free(t->src);
  clist_free(&t->require);
}

void target_print(target_t *t) {
  info(_("Target details:\n"));
  printf(FG_BOLD "    %s        " FG_RESET "=> %s\n", _("Name"), t->name);
  printf(FG_BOLD "    %s " FG_RESET "=> %s\n", _("Description"), t->desc);
  printf("\n");
}
